/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http//www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is ART Ontology API (Sesame2 Implementation).
 *
 * The Initial Developer of the Original Code is University of Roma Tor Vergata.
 * Portions created by University of Roma Tor Vergata are Copyright (C) 2007.
 * All Rights Reserved.
 *
 * ART Ontology API (Sesame2 Implementation) was developed by the Artificial Intelligence Research Group
 * (art.uniroma2.it) at the University of Roma Tor Vergata
 * Current information about the ART Ontology API (Sesame2 Implementation) can be obtained at 
 * http//art.uniroma2.it/owlart
 *
 */
package it.uniroma2.art.owlart.sesame2impl.models;

import it.uniroma2.art.owlart.exceptions.ModelAccessException;
import it.uniroma2.art.owlart.exceptions.ModelCreationException;
import it.uniroma2.art.owlart.exceptions.ModelUpdateException;
import it.uniroma2.art.owlart.exceptions.UnsupportedQueryLanguageException;
import it.uniroma2.art.owlart.exceptions.UnsupportedRDFFormatException;
import it.uniroma2.art.owlart.io.RDFFormat;
import it.uniroma2.art.owlart.model.ARTBNode;
import it.uniroma2.art.owlart.model.ARTLiteral;
import it.uniroma2.art.owlart.model.ARTNode;
import it.uniroma2.art.owlart.model.ARTResource;
import it.uniroma2.art.owlart.model.ARTStatement;
import it.uniroma2.art.owlart.model.ARTURIResource;
import it.uniroma2.art.owlart.model.NodeFilters;
import it.uniroma2.art.owlart.models.BaseRDFTripleModel;
import it.uniroma2.art.owlart.models.RDFSReasoner;
import it.uniroma2.art.owlart.models.TransactionBasedModel;
import it.uniroma2.art.owlart.models.impl.BaseRDFModelImpl;
import it.uniroma2.art.owlart.navigation.ARTNamespaceIterator;
import it.uniroma2.art.owlart.navigation.ARTResourceIterator;
import it.uniroma2.art.owlart.navigation.ARTStatementIterator;
import it.uniroma2.art.owlart.query.BooleanQuery;
import it.uniroma2.art.owlart.query.GraphQuery;
import it.uniroma2.art.owlart.query.MalformedQueryException;
import it.uniroma2.art.owlart.query.Query;
import it.uniroma2.art.owlart.query.QueryLanguage;
import it.uniroma2.art.owlart.query.TupleQuery;
import it.uniroma2.art.owlart.resources.Resources;
import it.uniroma2.art.owlart.sesame2impl.Sesame2ARTResourceFactory;
import it.uniroma2.art.owlart.sesame2impl.io.RDFFormatConverter;
import it.uniroma2.art.owlart.sesame2impl.navigation.Sesame2ARTNamespaceIteratorImpl;
import it.uniroma2.art.owlart.sesame2impl.navigation.Sesame2ARTResourceIteratorImpl;
import it.uniroma2.art.owlart.sesame2impl.navigation.Sesame2ARTStatementIteratorImpl;
import it.uniroma2.art.owlart.sesame2impl.query.BooleanQuerySesame2Impl;
import it.uniroma2.art.owlart.sesame2impl.query.GraphQuerySesame2Impl;
import it.uniroma2.art.owlart.sesame2impl.query.QueryLanguageConverter;
import it.uniroma2.art.owlart.sesame2impl.query.TupleQuerySesame2Impl;
import it.uniroma2.art.owlart.utilities.RDFIterators;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.openrdf.model.BNode;
import org.openrdf.model.Namespace;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.ValueFactory;
import org.openrdf.model.vocabulary.OWL;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.RepositoryResult;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.RDFWriter;
import org.openrdf.sail.SailException;

public class BaseRDFModelSesame2Impl extends BaseRDFModelImpl implements BaseRDFTripleModel, RDFSReasoner,
		TransactionBasedModel {

	protected static Logger logger = LoggerFactory.getLogger(BaseRDFModelSesame2Impl.class);

	protected Repository localrepository;
	// protected SailConnection conn;
	protected RepositoryConnection repConn;
	protected ValueFactory vF;
	protected String baseURI;
	protected String defaultNamespace;

	protected boolean rdfsReasoning;

	public BaseRDFModelSesame2Impl(Repository repo, boolean rdfsReasoning) throws SailException,
			RepositoryException, ModelCreationException {

		logger.info("initializing Sesame2 repository...");
		this.localrepository = repo;

		this.rdfsReasoning = rdfsReasoning;

		/*
		 * try { conn = localrepository.getSail().getConnection(); } catch(SailException e) { throw new
		 * ModelCreationException(e.getMessage()); }
		 */
		logger.info("getting Connection...");
		try {
			repConn = localrepository.getConnection();
		} catch (RepositoryException e) {
			throw new ModelCreationException(e.getMessage());
		}
		logger.info("setting autocommit to true...");
		repConn.setAutoCommit(true);
		
		logger.info("initializing value factory...");
		vF = repo.getValueFactory();
		
		logger.info("BaseRDFModel Sesame2 repository initialized...");
	}

	public void addStatement(ARTStatement stat, ARTResource... graphs) throws ModelUpdateException {
		try {
			repConn.add(Sesame2ARTResourceFactory.aRTStatement2SesameStatement(stat), getSesameContexts(true,
					graphs));
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		}
	}

	public void addTriple(ARTResource subject, ARTURIResource predicate, ARTNode object,
			ARTResource... graphs) throws ModelUpdateException {
		try {
			repConn.add(Sesame2ARTResourceFactory.aRTResource2SesameResource(subject),
					Sesame2ARTResourceFactory.aRTURIResource2SesameURI(predicate), Sesame2ARTResourceFactory
							.aRTNode2SesameValue(object), getSesameContexts(true, graphs));
			// logger.debug("adding triple: " + subject + "|" + predicate + "|" + object + "|" +
			// (graphs.length>0?"context0: "+graphs[0]:"nocontext"));
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		}
	}

	public void addRDF(File inputFile, String baseURI, RDFFormat rdfFormat, ARTResource... graphs)
			throws FileNotFoundException, IOException, ModelAccessException, ModelUpdateException,
			UnsupportedRDFFormatException {
		try {
			repConn.add(inputFile, baseURI, RDFFormatConverter.convert(rdfFormat), getSesameContexts(true,
					graphs));
		} catch (RDFParseException e) {
			logger.error("" + e);
			throw new IOException(e);
		} catch (RepositoryException e) {
			logger.error("" + e);
			throw new ModelUpdateException(e);
		}
	}

	public void addRDF(URL url, String baseURI, RDFFormat rdfFormat, ARTResource... graphs)
			throws FileNotFoundException, IOException, ModelAccessException, ModelUpdateException,
			UnsupportedRDFFormatException {
		try {
			URLConnection uricon = (URLConnection) url.openConnection();
			uricon.addRequestProperty("accept", rdfFormat.getMIMEType());
			InputStream instream = uricon.getInputStream();
			repConn.add(instream, baseURI, RDFFormatConverter.convert(rdfFormat), getSesameContexts(true,
					graphs));
		} catch (RDFParseException e) {
			logger.error("" + e);
			throw new IOException(e);
		} catch (RepositoryException e) {
			logger.error("" + e);
			throw new ModelUpdateException(e);
		}
	}

	public void deleteTriple(ARTResource subject, ARTURIResource property, ARTNode object,
			ARTResource... graphs) throws ModelUpdateException {
		try {
			repConn.remove(Sesame2ARTResourceFactory.aRTResource2SesameResource(subject),
					(URI) Sesame2ARTResourceFactory.aRTResource2SesameResource(property),
					Sesame2ARTResourceFactory.aRTNode2SesameValue(object), getSesameContexts(graphs));
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		}

	}

	public void deleteStatement(ARTStatement statement, ARTResource... graphs) throws ModelUpdateException {
		try {
			repConn.remove(Sesame2ARTResourceFactory.aRTResource2SesameResource(statement.getSubject()),
					(URI) Sesame2ARTResourceFactory.aRTResource2SesameResource(statement.getPredicate()),
					Sesame2ARTResourceFactory.aRTNode2SesameValue(statement.getObject()),
					getSesameContexts(graphs));
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e.getMessage());
		}
	}

	// this method implementation first checks if this is an OWL Model (we do not use instanceof since the
	// base repository typically encapsulated in an OWLModel is not an OWLModel).
	// then it clears all the data and then rechecks if the owl is still present (it may have been deleted
	// due to the graphs specified in graphs). If it is not, it reloads it
	public void clearRDF(ARTResource... graphs) throws ModelUpdateException {
		try {
			boolean isOWL = false;
			ARTURIResource owl = this.createURIResource(OWL.NAMESPACE);
			Collection<ARTResource> ngs = RDFIterators.getCollectionFromIterator(this.listNamedGraphs());
			if (ngs.contains(owl))
				isOWL = true;
			logger.debug("repository originally contained the OWL named graph: " + isOWL);

			repConn.clear(getSesameContexts(graphs));

			ngs = RDFIterators.getCollectionFromIterator(this.listNamedGraphs());
			if (isOWL && !ngs.contains(this.createURIResource(OWL.NAMESPACE))) {
				logger.debug("repository does not contain anymore the OWL named graph, reloading it");
				this.addRDF(Resources.class.getResource("owl.rdfs"), OWL.NAMESPACE, RDFFormat.RDFXML, owl);
			}

		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		} catch (ModelAccessException e) {
			throw new ModelUpdateException(e);
		} catch (FileNotFoundException e) {
			throw new ModelUpdateException(e);
		} catch (IOException e) {
			throw new ModelUpdateException(e);
		} catch (UnsupportedRDFFormatException e) {
			throw new ModelUpdateException(e);
		}
	}

	public String getBaseURI() {
		return baseURI;
	}

	public String getDefaultNamespace() {
		return defaultNamespace;
	}

	public ARTStatementIterator listStatements(ARTResource subj, ARTURIResource pred, ARTNode obj,
			boolean inferred, ARTResource... graphs) throws ModelAccessException {
		RepositoryResult<Statement> repRes;
		try {
			repRes = repConn.getStatements(Sesame2ARTResourceFactory.aRTResource2SesameResource(subj),
					Sesame2ARTResourceFactory.aRTURIResource2SesameURI(pred), Sesame2ARTResourceFactory
							.aRTNode2SesameValue(obj), inferred, getSesameContexts(graphs));
			return new Sesame2ARTStatementIteratorImpl(repRes);
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
	}

	public boolean hasTriple(ARTResource subj, ARTURIResource pred, ARTNode obj, boolean inferred,
			ARTResource... graphs) throws ModelAccessException {

		try {
			return repConn.hasStatement(Sesame2ARTResourceFactory.aRTResource2SesameResource(subj),
					(URI) Sesame2ARTResourceFactory.aRTURIResource2SesameURI(pred), Sesame2ARTResourceFactory
							.aRTNode2SesameValue(obj), inferred, getSesameContexts(graphs));
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
	}

	public boolean hasStatement(ARTStatement stat, boolean inferred, ARTResource... graphs)
			throws ModelAccessException {
		try {
			return repConn.hasStatement(Sesame2ARTResourceFactory.aRTStatement2SesameStatement(stat),
					inferred, getSesameContexts(graphs));
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
	}

	public ARTResourceIterator listNamedGraphs() throws ModelAccessException {
		RepositoryResult<Resource> nsIt;
		try {
			nsIt = repConn.getContextIDs();
			return (new Sesame2ARTResourceIteratorImpl(nsIt));
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
	}

	public ARTNamespaceIterator listNamespaces() throws ModelAccessException {
		RepositoryResult<Namespace> nsIt;
		try {
			nsIt = repConn.getNamespaces();
			return (new Sesame2ARTNamespaceIteratorImpl(nsIt));
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}

	}

	public void setBaseURI(String uri) throws ModelUpdateException {
		this.baseURI = uri;
	}

	public void setDefaultNamespace(String namespace) throws ModelUpdateException {
		this.defaultNamespace = namespace;
	}

	public void writeRDF(File outputFile, RDFFormat rdfFormat, ARTResource... graphs) throws IOException,
			ModelAccessException, UnsupportedRDFFormatException {
		FileOutputStream fos = new FileOutputStream(outputFile);
		RDFWriter writer = RDFFormatConverter.getWriterFactory(rdfFormat).getWriter(fos);

		try {
			repConn.export(writer, getSesameContexts(graphs));
		} catch (RepositoryException e) {
			logger.error("" + e);
			throw new ModelAccessException(e);
		} catch (RDFHandlerException e) {
			logger.error("" + e);
			throw new IOException(e);
		}
	}

	public void writeRDF(OutputStream os, RDFFormat rdfFormat, ARTResource... graphs) throws IOException,
			ModelAccessException, UnsupportedRDFFormatException {
		RDFWriter writer = RDFFormatConverter.getWriterFactory(rdfFormat).getWriter(os);

		try {
			repConn.export(writer, getSesameContexts(graphs));
		} catch (RepositoryException e) {
			logger.error("" + e);
			throw new ModelAccessException(e);
		} catch (RDFHandlerException e) {
			logger.error("" + e);
			throw new IOException(e);
		}
	}

	/*
	 * *************************************** FROM PrefixMapping Interface
	 * **************************************
	 */

	// TODO STARRED check if Sesame 2 has support for such methods
	public String expandQName(String qname) throws ModelAccessException {
		if (maybeIsURI(qname))
			return qname;
		String[] parts = qname.split(":");
		if (parts.length == 1)
			return getDefaultNamespace() + qname;
		else
			return getNSForPrefix(parts[0]) + parts[1];
	}

	// TODO URGENT!!! THIS IS TO DISCOVER QNAME WHICH ARE
	// ALREADY URI. but...put better method for
	// discovering if the qame is an uri!!!
	private boolean maybeIsURI(String input) {
		if (input.contains("#") || input.contains("/"))
			return true;
		return false;
	}

	public String getNSForPrefix(String prefix) throws ModelAccessException {
		try {
			return repConn.getNamespace(prefix);
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
	}

	public Map<String, String> getNamespacePrefixMapping() throws ModelAccessException {
		Map<String, String> res = new HashMap<String, String>();
		RepositoryResult<Namespace> resN;
		try {
			resN = repConn.getNamespaces();
			while (resN.hasNext()) {
				Namespace nm = resN.next();
				res.put(nm.getPrefix(), nm.getName());
			}
			resN.close();
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
		return res;
	}

	public String getPrefixForNS(String namespace) throws ModelAccessException {
		try {
			RepositoryResult<Namespace> res = repConn.getNamespaces();
			while (res.hasNext()) {
				Namespace nm = res.next();
				if (nm.getName().equalsIgnoreCase(namespace))
					return nm.getPrefix();
			}
			res.close();
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
		return null;
	}

	public String getQName(String uri) throws ModelAccessException {
		URI realURI = vF.createURI(uri);
		String namespace = realURI.getNamespace();
		if (namespace.equals(defaultNamespace))
			return realURI.getLocalName();
		String prefix = getPrefixForNS(namespace);
		if (prefix == null)
			return uri;
		else
			return prefix + ":" + realURI.getLocalName();
	}

	public void removeNsPrefixMapping(String namespace) throws ModelUpdateException {
		try {
			String prefix = getPrefixForNS(namespace);
			repConn.removeNamespace(prefix);
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		} catch (ModelAccessException e) {
			throw new ModelUpdateException(e);
		}
	}

	public void setNsPrefix(String namespace, String prefix) throws ModelUpdateException {
		try {
			String prevPrefix = getPrefixForNS(namespace);
			if (prevPrefix != null) {
				System.err.println("a mapping already exists for " + namespace + ", removing it");
				repConn.removeNamespace(prevPrefix);
			}
			repConn.setNamespace(prefix, namespace);
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		} catch (ModelAccessException e) {
			throw new ModelUpdateException(e);
		}
	}

	/*
	 * *************************************** Resource Creation methods
	 * **************************************
	 */

	public ARTLiteral createLiteral(String literalString) {
		return Sesame2ARTResourceFactory.sesameLiteral2ARTLiteral(vF.createLiteral(literalString));
	}

	public ARTLiteral createLiteral(String literalString, String language) {
		return Sesame2ARTResourceFactory.sesameLiteral2ARTLiteral(vF.createLiteral(literalString, language));
	}

	public ARTLiteral createLiteral(String literalString, ARTURIResource datatype) {
		return Sesame2ARTResourceFactory.sesameLiteral2ARTLiteral(vF.createLiteral(literalString,
				Sesame2ARTResourceFactory.aRTURIResource2SesameURI(datatype)));
	}

	public ARTURIResource createURIResource(String uri) {
		URI newURI = vF.createURI(uri);
		return Sesame2ARTResourceFactory.sesameURI2ARTURIResource(newURI);
	}

	public void close() throws ModelUpdateException {
		try {
			logger.info("closing owlModel");
			repConn.close();
			// conn.close();
			localrepository.shutDown();
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		}
		/*
		 * catch (SailException e) { throw new ModelUpdateException(e); }
		 */
	}

	public ARTBNode createBNode() {
		BNode bNode = vF.createBNode();
		return Sesame2ARTResourceFactory.sesameBNode2ARTBNode(bNode);
	}

	public ARTBNode createBNode(String ID) {
		BNode bNode = vF.createBNode(ID);
		return Sesame2ARTResourceFactory.sesameBNode2ARTBNode(bNode);
	}

	/*
	 * *************************************** QUERY Methods **************************************
	 */

	public BooleanQuery createBooleanQuery(QueryLanguage ql, String query, String baseURI)
			throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		try {
			return new BooleanQuerySesame2Impl(repConn.prepareBooleanQuery(
					QueryLanguageConverter.convert(ql), query, baseURI));
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		} catch (org.openrdf.query.MalformedQueryException e) {
			throw new MalformedQueryException(e);
		}
	}

	public GraphQuery createGraphQuery(QueryLanguage ql, String query, String baseURI)
			throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		try {
			return new GraphQuerySesame2Impl(repConn.prepareGraphQuery(QueryLanguageConverter.convert(ql),
					query, baseURI));
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		} catch (org.openrdf.query.MalformedQueryException e) {
			throw new MalformedQueryException(e);
		}
	}

	public Query createQuery(QueryLanguage ql, String query, String baseURI)
			throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		try {
			org.openrdf.query.Query ses2query = repConn.prepareQuery(QueryLanguageConverter.convert(ql),
					query, baseURI);
			if (ses2query instanceof org.openrdf.query.TupleQuery) {
				return new TupleQuerySesame2Impl((org.openrdf.query.TupleQuery) ses2query);
			} else if (ses2query instanceof org.openrdf.query.BooleanQuery)
				return new BooleanQuerySesame2Impl((org.openrdf.query.BooleanQuery) ses2query);
			else if (ses2query instanceof org.openrdf.query.GraphQuery)
				return new GraphQuerySesame2Impl((org.openrdf.query.GraphQuery) ses2query);
			else
				throw new ModelAccessException("unknown query type");
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		} catch (org.openrdf.query.MalformedQueryException e) {
			logger.error("malformed query");
			throw new MalformedQueryException(e);
		}
	}

	public TupleQuery createTupleQuery(QueryLanguage ql, String query, String baseURI)
			throws UnsupportedQueryLanguageException, ModelAccessException, MalformedQueryException {
		try {
			return new TupleQuerySesame2Impl(repConn.prepareTupleQuery(QueryLanguageConverter.convert(ql),
					query, baseURI));
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		} catch (org.openrdf.query.MalformedQueryException e) {
			throw new MalformedQueryException(e);
		}
	}

	/*
	 * *************************************** Local Utility Methods **************************************
	 */

	private Resource[] getSesameContexts(ARTResource... namedGraphs) {
		return getSesameContexts(false, namedGraphs);
	}

	private Resource[] getSesameContexts(boolean add, ARTResource... graphs) {
		if (graphs == null)
			throw new IllegalArgumentException();
		Resource[] resContexts = new Resource[graphs.length];
		for (int i = 0; i < graphs.length; i++) {
			if (graphs[i] == null)
				throw new IllegalArgumentException(); // a null value for a contest is not accepted in OWLArt
			// (though in Sesame it means: null-context), null-context (main unnamed graph) in OWLArt must be
			// specified as MAINGRAPH
			else if (graphs[i] == NodeFilters.ANY)
				// if any of the graphs is ANY, then an empty array is sent to Sesame (which interpretes as
				// all graphs when reading/deleting and maingraph when adding triples)
				// TODO could be implemented, when writing, as a special NodeFilter/graphs which always fails
				// the equals
				if (!add)
					return new Resource[0];
				else
					throw new IllegalArgumentException(
							"the ANY Node can be used only when reading/deleting triples from the model, not for writing");
			else if (graphs[i] == NodeFilters.MAINGRAPH)
				resContexts[i] = null; // in Sesame, MAINGRAPH is implemented with null on the graphs
			else
				resContexts[i] = Sesame2ARTResourceFactory.aRTResource2SesameResource(graphs[i]);
		}
		return resContexts;
	}

	public boolean supportsSubPropertyMaterialization() {
		// true if the forwardChaining Sail is put in the sail stack in the configuration in
		// ARTModelFactorySesame2Impl
		return rdfsReasoning;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see it.uniroma2.art.owlart.models.TransactionBasedModel#commit()
	 */
	public void commit() throws ModelUpdateException {
		try {
			repConn.commit();
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see it.uniroma2.art.owlart.models.TransactionBasedModel#isAutoCommit()
	 */
	public boolean isAutoCommit() throws ModelAccessException {
		try {
			return repConn.isAutoCommit();
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see it.uniroma2.art.owlart.models.TransactionBasedModel#rollBack()
	 */
	public void rollBack() throws ModelAccessException {
		try {
			repConn.rollback();
		} catch (RepositoryException e) {
			throw new ModelAccessException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see it.uniroma2.art.owlart.models.TransactionBasedModel#setAutoCommit()
	 */
	public void setAutoCommit(boolean value) throws ModelUpdateException {
		try {
			repConn.setAutoCommit(value);
		} catch (RepositoryException e) {
			throw new ModelUpdateException(e);
		}
	}

}
